import { RequestOptionsType } from './types/general';
import { DefaultQueryParams, OrderImpactBodyParams, PerformanceInformationQueryParams } from './types/params';
import { AccountPositionsResponseType, AccountResponseType, AccountsResponseType, BalanceResponseType, BrokerageAuthorizationTypeObjectResponseType, BrokerageAuthResponseType, BrokerageResponseType, CurrencyResponseType, DeleteUserResponseType, ExchangeRateResponseType, MultipleTradesOrderImpactResponseType, OrderImpactResponseType, OrderResponseType, PortfolioGroupPositionsResponseType, PortfolioGroupResponseType, RedirectURIResponseType, RegisterUserResponseType, SecurityTypeResponseType, StockExchangeResponseType, SymbolsQuoteResponseType, TransactionHistoryResponseType, UniversalSymbolResponseType, UserHoldingsResponseType, RetrieveJWTResponseType, AccountHoldingsResponseType, PerformanceInformationResponseType, PartnerDataResponseType, CancelOrderResponseType, PlaceOrderResponseType, DeleteAuthorizationResponseType, BrokerageAuthsResponseType, UsersResponseType } from './types/response';
/**
 * @class SnapTradeFetch
 */
export declare class SnapTradeFetch {
    clientId: string;
    consumerKey: string;
    /**
     * Creates an instance of SnapTrade fetch.
     * @param {string} clientId - SnapTrade Client ID (generated and provided to partner by Passiv)
     * @param {string} consumerKey - SnapTrade Consumer Key (generated and provided to partner by Passiv)
     */
    constructor(clientId: string, consumerKey: string);
    checkDeprecated(): void;
    /** Helper Functions */
    /**
     * Generate RSA private/public key
     * @param {string} path - path to store the private key
     * @returns string
     */
    generateRSA(path: string): string;
    /**
     * Decrypt encrypted shared key
     * @param {string} privateKeyFilePath - path to retrieve the private key
     * @param {string} encryptedSharedKey - encrypted shared key
     * @returns string
     */
    decryptSharedKey(privateKeyFilePath: string, encryptedSharedKey: string): string;
    /**
     * Decrypt encrypted message
     * @param {string} sharedKey
     * @param {encryptedMessage: string, tag: string, nonce: string} encryptedMessageData
     * @returns string
     */
    decryptMessage(sharedKey: string, encryptedMessageData: {
        encryptedMessage: string;
        tag: string;
        nonce: string;
    }): string;
    /** Authentication **/
    /**
     * Retrieve encrypted JWT token
     * @param {DefaultQueryParams} defaultQueryParams
     * @returns  Promise<RetrieveJWTResponseType>
     */
    retrieveJWT({ userId, userSecret, }: DefaultQueryParams): Promise<RetrieveJWTResponseType>;
    /**
     * Register user with SnapTrade
     * in order to create secure brokerage authorizations.
     * @param {userId: string, rsaPublicKey?: string} data
     * @returns Promise<RegisterUserResponseType>
     */
    registerUser(data: {
        userId: string;
        rsaPublicKey?: string;
    }): Promise<RegisterUserResponseType>;
    /**
     * Delete user, disabling all brokerage
     * authorizations and permanently deleting all data associated with the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @returns Promise<DeleteUserResponseType>
     */
    deleteUser({ userId, userSecret, }: DefaultQueryParams): Promise<DeleteUserResponseType>;
    /**
     * Generate a redirect URI to securely login a user to the SnapTrade Connection Portal.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {broker: string, immediateRedirect: boolean, customRedirect: string, reconnect: string} data
     * @returns Promise<RedirectURIResponseType>
     */
    generateRedirectURI({ userId, userSecret }: DefaultQueryParams, data?: {
        broker?: string;
        immediateRedirect?: boolean;
        customRedirect?: string;
        reconnect?: string;
        connectionType?: string;
        connectionPortalVersion?: string;
    }): Promise<RedirectURIResponseType>;
    /**
     * Get a list of all SnapTrade users that partner has registered on their platform
     * @returns Promise<UsersResponseType>
     */
    listUsers(): Promise<UsersResponseType>;
    /** Account Information **/
    /**
     * List all accounts for the user, plus balances and positions for each account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param { authorizationIds: string[] } extraParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<UserHoldingsResponseType>
     */
    fetchUserHoldings({ userId, userSecret }: DefaultQueryParams, extraParams?: {
        authorizationIds: string[];
    }, options?: RequestOptionsType): Promise<UserHoldingsResponseType>;
    /**
     * Get a specific account for the user, plus balances and positions for that account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param accountId: string
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountHoldingsResponseType>
     */
    fetchAccountHoldings({ userId, userSecret }: DefaultQueryParams, accountId: string, options?: RequestOptionsType): Promise<AccountHoldingsResponseType>;
    /**
     * List all investment accounts for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountsResponseType>
     */
    fetchUserAccounts({ userId, userSecret }: DefaultQueryParams, options?: RequestOptionsType): Promise<AccountsResponseType>;
    /**
     * Return details of a specific investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountResponseType>
     */
    fetchAccount({ userId, userSecret }: DefaultQueryParams, accountId: string, options?: RequestOptionsType): Promise<AccountResponseType>;
    /**
     * Get all cash balances of an investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId\
     * @param {RequestOptionsType} [options]
     * @returns Promise<BalanceResponseType>
    
     */
    fetchAccountBalances({ userId, userSecret }: DefaultQueryParams, accountId: string, options?: RequestOptionsType): Promise<BalanceResponseType>;
    /**
     * Get all positions of an investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountPositionsResponseType>
     */
    fetchAccountPositions({ userId, userSecret }: DefaultQueryParams, accountId: string, options?: RequestOptionsType): Promise<AccountPositionsResponseType>;
    /** Trading **/
    /**
     * Get all history of orders placed in account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param { status: string; days: number } extraParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<OrdersResponseType>
     */
    fetchOrdersHistory({ userId, userSecret }: DefaultQueryParams, accountId: string, extraParams?: {
        status?: string;
        days?: number;
    }, options?: RequestOptionsType): Promise<OrderResponseType>;
    /**
     * Cancel open order in account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {brokerage_order_id: string} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<CancelOrderResponseType>
     */
    cancelOpenOrder({ userId, userSecret }: DefaultQueryParams, accountId: string, data: {
        brokerage_order_id: string;
    }, options?: RequestOptionsType): Promise<CancelOrderResponseType>;
    /**
     * Get symbols quote.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<SymbolsQuoteResponseType>
     */
    fetchSymbolsQuote({ userId, userSecret }: DefaultQueryParams, accountId: string, extraParams: {
        symbols: string;
        use_ticker?: boolean;
    }, options?: RequestOptionsType): Promise<SymbolsQuoteResponseType>;
    /**
     * Return the impact of placing a series of trades on the portfolio
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} portfolioGroupId
     * @param {string} calculatedTradeId
     * @param {RequestOptionsType} [options]
     * @returns Promise<MultipleTradesOrderImpactResponseType>
     */
    multipleTradesOrderImpact({ userId, userSecret }: DefaultQueryParams, portfolioGroupId: string, calculatedTradeId: string, options?: RequestOptionsType): Promise<MultipleTradesOrderImpactResponseType>;
    /**
     * Check impact of trades on account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {OrderImpactBodyParams} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<OrderImpactResponseType>
     */
    orderImpact({ userId, userSecret }: DefaultQueryParams, data: OrderImpactBodyParams, options?: RequestOptionsType): Promise<OrderImpactResponseType>;
    /**
     * Place order.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} tradeId
     * @param {RequestOptionsType} [options]
     * @returns Promise<PlaceOrderResponseType>
     */
    placeOrder({ userId, userSecret }: DefaultQueryParams, tradeId: string, options?: RequestOptionsType): Promise<PlaceOrderResponseType>;
    /**
     * Place a trade with NO validation.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {OrderImpactBodyParams} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<PlaceOrderResponseType>
     */
    placeTradeWithNoValidation({ userId, userSecret }: DefaultQueryParams, data: OrderImpactBodyParams, options?: RequestOptionsType): Promise<PlaceOrderResponseType>;
    /** Connections **/
    /**
     * List all brokerage authorizations for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<BrokerageAuthsResponseType>
     */
    fetchBrokerageAuthorizations({ userId, userSecret }: DefaultQueryParams, options?: RequestOptionsType): Promise<BrokerageAuthsResponseType>;
    /**
     * Get detail of a specific brokerage authorizations for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} authorizationId
     * @param {RequestOptionsType} [options]
     * @returns Promise<BrokerageAuthResponseType>
     */
    fetchAuthorization({ userId, userSecret }: DefaultQueryParams, authorizationId: string, options?: RequestOptionsType): Promise<BrokerageAuthResponseType>;
    /**
     * Remove a brokerage authorization.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} authorizationId
     * @param {RequestOptionsType} [options]
     * @returns Promise<DeleteAuthorizationResponseType>
     */
    deleteAuthorization({ userId, userSecret }: DefaultQueryParams, authorizationId: string, options?: RequestOptionsType): Promise<DeleteAuthorizationResponseType>;
    /** Reference Data **/
    /**
     * List of all supported brokerages.
     * @returns Promise<BrokerageResponseType>
     */
    fetchBrokerages(): Promise<BrokerageResponseType>;
    /**
     * List of all supported currencies.
     * @returns Promise<CurrencyResponseType>
     */
    fetchCurrencies(): Promise<CurrencyResponseType>;
    /**
     * Return the exchange rates of all supported currencies.
     * @returns Promise<ExchangeRateResponseType>
     */
    fetchExchangeCurrencies(): Promise<ExchangeRateResponseType>;
    /**
     * Return the exchange rate of a currency pair.
     * @param {string} currencyPair
     * @returns Promise<ExchangeRateType>
     */
    getCurrencyPair(currencyPair: string): Promise<ExchangeRateResponseType>;
    /**
     * Search for symbols.
     * @param {substring: string} data
     * @returns Promise<UniversalSymbolResponseType>
     */
    searchSymbols(data: {
        substring: string;
    }): Promise<UniversalSymbolResponseType>;
    /**
     * Get details of a symbol.
     * @param {string} symbolId
     * @returns Promise<UniversalSymbolType>
     */
    getSymbolDetailById(symbolId: string): Promise<UniversalSymbolResponseType>;
    /**
     * Get details of a symbol by the ticker.
     * @param {string} ticker
     * @returns Promise<UniversalSymbolResponseType>
     */
    getSymbolDetailByTicker(ticker: string): Promise<UniversalSymbolResponseType>;
    /**
     * Get a list of all defined security types
     * @returns Promise<SecurityTypeResponseType>
     */
    fetchListOfSecurityTypes(): Promise<SecurityTypeResponseType>;
    /**
     * Get a list of all defined Brokerage Authorization Type objects
     * @param { brokerage: string[] } extraParams
     * @returns Promise<BrokerageAuthorizationTypeObjectResponseType>
     */
    fetchListOfBrokerageAuthorizationTypes(extraParams: {
        brokerage: string[];
    }): Promise<BrokerageAuthorizationTypeObjectResponseType>;
    /**
     * Get a list of stock exchanges and their suffixes
     * @returns Promise<StockExchangeResponseType>
     */
    fetchListOfStockExchanges(): Promise<StockExchangeResponseType>;
    /**
     * Get data relevant to the partner
     * @returns Promise<PartnerDataResponseType>
     */
    partnerData(): Promise<PartnerDataResponseType>;
    /** Portfolio Management **/
    /**
     * List of all portfolio groups.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<PortfolioGroupResponseType>
     */
    fetchPortfolioGroups({ userId, userSecret }: DefaultQueryParams, options?: RequestOptionsType): Promise<PortfolioGroupResponseType>;
    /**
     * List Portfolio Group positions.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param portfolioGroupId
     * @param {RequestOptionsType} [options]
     * @returns Promise<PortfolioGroupPositionsResponseType>
     */
    fetchPortfolioGroupPrositions({ userId, userSecret }: DefaultQueryParams, portfolioGroupId: string, options?: RequestOptionsType): Promise<PortfolioGroupPositionsResponseType>;
    /** Reporting **/
    /**
     * Get transaction history for a user
     * @param {DefaultQueryParams} defaultQueryParams
     * @param { startDate: string; endDate: string, accounts: string } extraParams
     * @returns Promise<TransactionHistoryResponseType>
     */
    fetchTransactionHistory({ userId, userSecret }: DefaultQueryParams, extraParams?: {
        startDate?: string;
        endDate?: string;
        accounts?: string;
        brokerageAuthorizations?: string[];
    }, options?: RequestOptionsType): Promise<TransactionHistoryResponseType>;
    /**
     * Get performance information for a specific time frame
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {PerformanceInformationQueryParams} extraParams
     * @returns Promise<PerformanceInformationResponseType>
     */
    fetchPerformanceInformation({ userId, userSecret }: DefaultQueryParams, extraParams: PerformanceInformationQueryParams, options?: RequestOptionsType): Promise<PerformanceInformationResponseType>;
}
