"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapTradeFetch = void 0;
const request_1 = require("./request");
const crypto_1 = require("crypto");
const fs = require("fs");
const NodeRSA = require("node-rsa");
/**
 * @class SnapTradeFetch
 */
class SnapTradeFetch {
    clientId;
    consumerKey;
    /**
     * Creates an instance of SnapTrade fetch.
     * @param {string} clientId - SnapTrade Client ID (generated and provided to partner by Passiv)
     * @param {string} consumerKey - SnapTrade Consumer Key (generated and provided to partner by Passiv)
     */
    constructor(clientId, consumerKey) {
        this.clientId = clientId;
        this.consumerKey = consumerKey;
        this.checkDeprecated();
    }
    checkDeprecated() {
        console.warn('This SDK has been deprecated and is no longer receiving support. Please use our maintained SDK here: https://www.npmjs.com/package/snaptrade-typescript-sdk');
    }
    /** Helper Functions */
    /**
     * Generate RSA private/public key
     * @param {string} path - path to store the private key
     * @returns string
     */
    generateRSA(path) {
        const key = new NodeRSA({ b: 4096 });
        const publicKey = key.exportKey('openssh-public');
        const privateKey = key.exportKey('pkcs8-private-pem');
        try {
            fs.writeFileSync(path, JSON.stringify({ privateKey }));
            return publicKey;
        }
        catch (err) {
            return JSON.stringify(err);
        }
    }
    /**
     * Decrypt encrypted shared key
     * @param {string} privateKeyFilePath - path to retrieve the private key
     * @param {string} encryptedSharedKey - encrypted shared key
     * @returns string
     */
    decryptSharedKey(privateKeyFilePath, encryptedSharedKey) {
        try {
            const rawFile = fs.readFileSync(privateKeyFilePath);
            const privateKey = JSON.parse(rawFile.toString()).privateKey;
            if (privateKey) {
                const buffer = Buffer.from(encryptedSharedKey, 'base64');
                const decryptedKey = (0, crypto_1.privateDecrypt)({
                    key: privateKey.toString(),
                    padding: crypto_1.constants.RSA_PKCS1_OAEP_PADDING,
                }, buffer);
                return decryptedKey.toString();
            }
            else {
                throw 'Cannot find the private key. Please check the path provided and make sure the JSON file with a privateKey exists.';
            }
        }
        catch (err) {
            return JSON.stringify(err);
        }
    }
    /**
     * Decrypt encrypted message
     * @param {string} sharedKey
     * @param {encryptedMessage: string, tag: string, nonce: string} encryptedMessageData
     * @returns string
     */
    decryptMessage(sharedKey, encryptedMessageData) {
        let iv = Buffer.from(encryptedMessageData.nonce).toString('base64');
        let encryptedText = Buffer.from(encryptedMessageData.encryptedMessage).toString('base64');
        // @ts-ignore
        let decipher = (0, crypto_1.createDecipheriv)('aes-256-ocb', Buffer.from(sharedKey), iv, {
            authTagLength: 32,
        });
        // @ts-ignore
        let decrypted = decipher.update(encryptedText);
        // @ts-ignore
        decrypted = Buffer.concat([decrypted, decipher.final()]);
        return decrypted.toString();
    }
    /** Authentication **/
    /**
     * Retrieve encrypted JWT token
     * @param {DefaultQueryParams} defaultQueryParams
     * @returns  Promise<RetrieveJWTResponseType>
     */
    async retrieveJWT({ userId, userSecret, }) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/snapTrade/encryptedJWT',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Register user with SnapTrade
     * in order to create secure brokerage authorizations.
     * @param {userId: string, rsaPublicKey?: string} data
     * @returns Promise<RegisterUserResponseType>
     */
    async registerUser(data) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/snapTrade/registerUser',
            method: 'post',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
            data,
        });
        return response;
    }
    /**
     * Delete user, disabling all brokerage
     * authorizations and permanently deleting all data associated with the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @returns Promise<DeleteUserResponseType>
     */
    async deleteUser({ userId, userSecret, }) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/snapTrade/deleteUser',
            method: 'post',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Generate a redirect URI to securely login a user to the SnapTrade Connection Portal.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {broker: string, immediateRedirect: boolean, customRedirect: string, reconnect: string} data
     * @returns Promise<RedirectURIResponseType>
     */
    async generateRedirectURI({ userId, userSecret }, data) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/snapTrade/login',
            method: 'post',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            data,
        });
        return response;
    }
    /**
     * Get a list of all SnapTrade users that partner has registered on their platform
     * @returns Promise<UsersResponseType>
     */
    async listUsers() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/snapTrade/listUsers',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /** Account Information **/
    /**
     * List all accounts for the user, plus balances and positions for each account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param { authorizationIds: string[] } extraParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<UserHoldingsResponseType>
     */
    async fetchUserHoldings({ userId, userSecret }, extraParams, options) {
        const extraParamsToString = {
            brokerage_authorizations: extraParams?.authorizationIds.toString(),
        };
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/holdings',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            extraParams: extraParamsToString.brokerage_authorizations
                ? extraParamsToString
                : null,
        });
        return response;
    }
    /**
     * Get a specific account for the user, plus balances and positions for that account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param accountId: string
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountHoldingsResponseType>
     */
    async fetchAccountHoldings({ userId, userSecret }, accountId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/holdings`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * List all investment accounts for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountsResponseType>
     */
    async fetchUserAccounts({ userId, userSecret }, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/accounts',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Return details of a specific investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountResponseType>
     */
    async fetchAccount({ userId, userSecret }, accountId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Get all cash balances of an investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId\
     * @param {RequestOptionsType} [options]
     * @returns Promise<BalanceResponseType>
    
     */
    async fetchAccountBalances({ userId, userSecret }, accountId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/balances`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Get all positions of an investment account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<AccountPositionsResponseType>
     */
    async fetchAccountPositions({ userId, userSecret }, accountId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/positions`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /** Trading **/
    /**
     * Get all history of orders placed in account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param { status: string; days: number } extraParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<OrdersResponseType>
     */
    async fetchOrdersHistory({ userId, userSecret }, accountId, extraParams, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/orders`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            extraParams,
        });
        return response;
    }
    /**
     * Cancel open order in account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {brokerage_order_id: string} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<CancelOrderResponseType>
     */
    async cancelOpenOrder({ userId, userSecret }, accountId, data, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/orders/cancel`,
            method: 'post',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            data,
        });
        return response;
    }
    /**
     * Get symbols quote.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} accountId
     * @param {RequestOptionsType} [options]
     * @returns Promise<SymbolsQuoteResponseType>
     */
    async fetchSymbolsQuote({ userId, userSecret }, accountId, extraParams, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/accounts/${accountId}/quotes`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            extraParams,
        });
        return response;
    }
    /**
     * Return the impact of placing a series of trades on the portfolio
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} portfolioGroupId
     * @param {string} calculatedTradeId
     * @param {RequestOptionsType} [options]
     * @returns Promise<MultipleTradesOrderImpactResponseType>
     */
    async multipleTradesOrderImpact({ userId, userSecret }, portfolioGroupId, calculatedTradeId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/portfolioGroups/${portfolioGroupId}/calculatedtrades/${calculatedTradeId}`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Check impact of trades on account.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {OrderImpactBodyParams} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<OrderImpactResponseType>
     */
    async orderImpact({ userId, userSecret }, data, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/trade/impact',
            method: 'post',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            data,
        });
        return response;
    }
    /**
     * Place order.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} tradeId
     * @param {RequestOptionsType} [options]
     * @returns Promise<PlaceOrderResponseType>
     */
    async placeOrder({ userId, userSecret }, tradeId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/trade/${tradeId}`,
            method: 'post',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Place a trade with NO validation.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {OrderImpactBodyParams} data
     * @param {RequestOptionsType} [options]
     * @returns Promise<PlaceOrderResponseType>
     */
    async placeTradeWithNoValidation({ userId, userSecret }, data, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/trade/place',
            method: 'post',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            data,
        });
        return response;
    }
    /** Connections **/
    /**
     * List all brokerage authorizations for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<BrokerageAuthsResponseType>
     */
    async fetchBrokerageAuthorizations({ userId, userSecret }, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/authorizations',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Get detail of a specific brokerage authorizations for the user.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} authorizationId
     * @param {RequestOptionsType} [options]
     * @returns Promise<BrokerageAuthResponseType>
     */
    async fetchAuthorization({ userId, userSecret }, authorizationId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/authorizations/${authorizationId}`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * Remove a brokerage authorization.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {string} authorizationId
     * @param {RequestOptionsType} [options]
     * @returns Promise<DeleteAuthorizationResponseType>
     */
    async deleteAuthorization({ userId, userSecret }, authorizationId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/authorizations/${authorizationId}`,
            method: 'delete',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /** Reference Data **/
    /**
     * List of all supported brokerages.
     * @returns Promise<BrokerageResponseType>
     */
    async fetchBrokerages() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/brokerages',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * List of all supported currencies.
     * @returns Promise<CurrencyResponseType>
     */
    async fetchCurrencies() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/currencies',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Return the exchange rates of all supported currencies.
     * @returns Promise<ExchangeRateResponseType>
     */
    async fetchExchangeCurrencies() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/currencies/rates',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Return the exchange rate of a currency pair.
     * @param {string} currencyPair
     * @returns Promise<ExchangeRateType>
     */
    async getCurrencyPair(currencyPair) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/currencies/rates/${currencyPair}`,
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Search for symbols.
     * @param {substring: string} data
     * @returns Promise<UniversalSymbolResponseType>
     */
    async searchSymbols(data) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/symbols',
            method: 'post',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
            data,
        });
        return response;
    }
    /**
     * Get details of a symbol.
     * @param {string} symbolId
     * @returns Promise<UniversalSymbolType>
     */
    async getSymbolDetailById(symbolId) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/symbols/${symbolId}`,
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Get details of a symbol by the ticker.
     * @param {string} ticker
     * @returns Promise<UniversalSymbolResponseType>
     */
    async getSymbolDetailByTicker(ticker) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/symbols/${ticker}`,
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Get a list of all defined security types
     * @returns Promise<SecurityTypeResponseType>
     */
    async fetchListOfSecurityTypes() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/securityTypes',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Get a list of all defined Brokerage Authorization Type objects
     * @param { brokerage: string[] } extraParams
     * @returns Promise<BrokerageAuthorizationTypeObjectResponseType>
     */
    async fetchListOfBrokerageAuthorizationTypes(extraParams) {
        const extraParamsToString = {
            brokerage: extraParams.brokerage.toString(),
        };
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/brokerageAuthorizationTypes',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
            extraParams: extraParamsToString,
        });
        return response;
    }
    /**
     * Get a list of stock exchanges and their suffixes
     * @returns Promise<StockExchangeResponseType>
     */
    async fetchListOfStockExchanges() {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/exchanges',
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /**
     * Get data relevant to the partner
     * @returns Promise<PartnerDataResponseType>
     */
    async partnerData() {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/snapTrade/partners/`,
            method: 'get',
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
            },
        });
        return response;
    }
    /** Portfolio Management **/
    /**
     * List of all portfolio groups.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {RequestOptionsType} [options]
     * @returns Promise<PortfolioGroupResponseType>
     */
    async fetchPortfolioGroups({ userId, userSecret }, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/portfolioGroups',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /**
     * List Portfolio Group positions.
     * @param {DefaultQueryParams} defaultQueryParams
     * @param portfolioGroupId
     * @param {RequestOptionsType} [options]
     * @returns Promise<PortfolioGroupPositionsResponseType>
     */
    async fetchPortfolioGroupPrositions({ userId, userSecret }, portfolioGroupId, options) {
        const response = await (0, request_1.request)({
            endpoint: `/api/v1/portfolioGroups/${portfolioGroupId}/positions`,
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
        });
        return response;
    }
    /** Reporting **/
    /**
     * Get transaction history for a user
     * @param {DefaultQueryParams} defaultQueryParams
     * @param { startDate: string; endDate: string, accounts: string } extraParams
     * @returns Promise<TransactionHistoryResponseType>
     */
    async fetchTransactionHistory({ userId, userSecret }, extraParams, options) {
        const modifiedExtraParams = {
            ...extraParams,
            brokerageAuthorizations: extraParams?.brokerageAuthorizations?.toString(),
        };
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/activities',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            extraParams: modifiedExtraParams,
        });
        return response;
    }
    /**
     * Get performance information for a specific time frame
     * @param {DefaultQueryParams} defaultQueryParams
     * @param {PerformanceInformationQueryParams} extraParams
     * @returns Promise<PerformanceInformationResponseType>
     */
    async fetchPerformanceInformation({ userId, userSecret }, extraParams, options) {
        const response = await (0, request_1.request)({
            endpoint: '/api/v1/performance/custom',
            method: 'get',
            timeout: options?.timeout,
            consumerKey: this.consumerKey,
            defaultQueryParams: {
                clientId: this.clientId,
                userSecret,
                userId,
            },
            extraParams,
        });
        return response;
    }
}
exports.SnapTradeFetch = SnapTradeFetch;
